@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-import numpy as np
-
 from npbrain.core import integrate
 from npbrain.core.neuron import *
 
@@ -48,26 +46,11 @@ def LIF(geometry, method=None, tau=10., Vr=0., Vth=10., noise=0., ref=0., name='
 
     judge_spike = get_spike_judger()
 
-    @integrate(method=method, noise=noise / tau)
+    @integrate(method=method, noise=noise / tau, signature='f8[:](f8[:], f8, f8[:])')
     def int_f(V, t, Isyn):
         return (-V + Vr + Isyn) / tau
 
     if ref > 0.:
-        # def update_state(neu_state, t):
-        #     # calculate states
-        #     not_in_ref = (t - neu_state[-2]) > ref
-        #     neu_state[-5] = not_in_ref
-        #     V = neu_state[0]
-        #     Isyn = neu_state[-1]
-        #     V_new = int_f(V, t, Isyn)
-        #     in_ref_idx = np.where(not_in_ref < 1.)[0]
-        #     V_new[in_ref_idx] = V[in_ref_idx]
-        #     neu_state[0] = V_new
-        #     # get spikes
-        #     spike_idx = judge_spike(neu_state, Vth, t)
-        #     neu_state[0][spike_idx] = Vr
-        #     neu_state[-5][spike_idx] = 0.
-
         def update_state(neu_state, t):
             V_new = int_f(neu_state[0], t, neu_state[-1])
             for idx in range(num):
