@@ -45,8 +45,8 @@ def STP(pre, post, weights, connection, U=0.15, tau_f=1500., tau_d=200.,
         The post-synaptic neuron group.
     weights : dict, np.ndarray, int, float
         The weighted coefficients of synapses.
-    connection : dict, str, callable
-        The connection method.
+    connection : tuple
+        The connection.
     delay : None, float
         The delay time length.
     tau_d : float
@@ -73,58 +73,70 @@ def STP(pre, post, weights, connection, U=0.15, tau_f=1500., tau_d=200.,
     .. [1] Tsodyks, Misha, Klaus Pawelzik, and Henry Markram. "Neural networks
            with dynamic synapses." Neural computation 10.4 (1998): 821-835.
     """
-    num_pre = pre.num
-    num_post = post.num
-    var2index = {'u': (2, 0), 'x': (2, 1)}
+    var2index = {'u': (0, 0), 'x': (0, 1)}
 
     pre_ids, post_ids, anchors = connection
     num = len(pre_ids)
-
-    # The first (num_syn, ) shape variable is "u"
-    # The second (num_syn, ) shape variable is "x"
-    state = initial_syn_state(delay, num_pre, num_post, num, num_syn_shape_var=2)
-    state[2][0] = np.ones(num) * u0
-    state[2][1] = np.ones(num) * x0
+    num_pre, num_post = pre.num, post.num
+    state = initial_syn_state(delay, num_syn=num, num_post=num_post, num_syn_var=2)
+    state[0][0] = np.ones(num) * u0
+    state[0][1] = np.ones(num) * x0
 
     clip = get_clip()
 
-    @integrate
+    # weights
+    if np.size(weights) == 1:
+        weights = np.ones(num) * weights
+    assert np.size(weights) == num, 'Unknown weights shape: {}'.format(weights.shape)
+
+    @integrate(signature='f8[:](f8[:], f8)')
     def int_u(u, t):
         return - u / tau_f
 
-    @integrate
+    @integrate(signature='f8[:](f8[:], f8)')
     def int_x(x, t):
         return (1 - x) / tau_d
 
-    @syn_delay
-    def update_state(syn_state, t):
+    def update_state(syn_state, t, delay_idx, pre_state, post_state):
         # get synapse state
-        u_old = syn_state[2][0]
-        x_old = syn_state[2][1]
-        pre_spike = syn_state[0][0]
+        u_old = syn_state[0][0]
+        x_old = syn_state[0][1]
+        pre_spike = pre_state[-3]
         # calculate synaptic state
-        spike_idx = np.where(pre_spike > 0.)[0]
         u_new = int_u(u_old, t)
         x_new = int_x(x_old, t)
-        for i in spike_idx:
-            idx = anchors[:, i]
-            u_new[idx[0]: idx[1]] += U * (1 - u_old[idx[0]: idx[1]])
-            x_new[idx[0]: idx[1]] -= u_new[idx[0]: idx[1]] * x_old[idx[0]: idx[1]]
+        for i in range(num_pre):
+            if pre_spike[i] > 0.:
+                se = anchors[:, i]
+                u_new[se[0]: se[1]] += U * (1 - u_old[se[0]: se[1]])
+                x_new[se[0]: se[1]] -= u_new[se[0]: se[1]] * x_old[se[0]: se[1]]
         u_new = clip(u_new, 0., 1.)
         x_new = clip(x_new, 0., 1.)
-        syn_state[2][0] = u_new
-        syn_state[2][1] = x_new
+        syn_state[0][0] = u_new
+        syn_state[0][1] = x_new
         # get post-synaptic values
         g = np.zeros(num_post)
         for i in range(num_pre):
-            idx = anchors[:, i]
-            post_idx = post_ids[idx[0]: idx[1]]
-            g[post_idx] += u_new[idx[0]: idx[1]] * x_new[idx[0]: idx[1]]
-        return g
+            se = anchors[:, i]
+            weight = weights[se[0]: se[1]]
+            u = u_new[se[0]: se[1]]
+            x = x_new[se[0]: se[1]]
+            post_idx = post_ids[se[0]: se[1]]
+            g[post_idx] += u * x * weight
+        syn_state[1][delay_idx] = g
 
-    def output_synapse(syn_state, var_index, post_neu_state):
-        output_idx = var_index[-2]
-        syn_val = syn_state[output_idx[0]][output_idx[1]]
-        post_neu_state[-1] += syn_val * weights
+    if hasattr(post, 'ref') and getattr(post, 'ref') > 0.:
+
+        def output_synapse(syn_state, output_idx, pre_state, post_state):
+            syn_val = syn_state[1][output_idx]
+            for idx in range(num_post):
+                val = syn_val[idx] * post_state[-5, idx]
+                post_state[-1, idx] += val
+
+    else:
+
+        def output_synapse(syn_state, output_idx, pre_state, post_state):
+            syn_val = syn_state[1][output_idx]
+            post_state[-1] += syn_val
 
     return Synapses(**locals())
