@@ -55,8 +55,9 @@ __all__ = [
 _backend = 'numpy'
 _nopython = True
 _fastmath = True
+_nogil = False
 _parallel = False
-_cache = False
+predefine_signature = True
 
 
 def set_backend(bk):
@@ -110,18 +111,21 @@ def set_numba(**kwargs):
                     ``parallel``, ``nopython``.
     :type kwargs: dict
     """
-    if 'cache' in kwargs:
-        global _cache
-        _cache = kwargs.pop('cache')
+    global _fastmath
+    global _nopython
+    global _nogil
+    global _parallel
+    global predefine_signature
+
     if 'fastmath' in kwargs:
-        global _fastmath
         _fastmath = kwargs.pop('fastmath')
-    if 'parallel' in kwargs:
-        global _parallel
-        _parallel = kwargs.pop('parallel')
     if 'nopython' in kwargs:
-        global _nopython
         _nopython = kwargs.pop('nopython')
+    if 'pre_sig' in kwargs:
+        predefine_signature = kwargs.pop('pre_sig')
+    if 'parallel' in kwargs:
+        _parallel = kwargs.pop('parallel')
+        _nogil = _parallel
 
 
 def get_numba_profile():
@@ -130,10 +134,12 @@ def get_numba_profile():
     :return: Settings.
     :rtype: dict
     """
-    return {'cache': _cache,
-            'fastmath': _fastmath,
-            'parallel': _parallel,
-            'nopython': _nopython}
+    return {
+        'fastmath': _fastmath,
+        'nopython': _nopython,
+        'nogil': _nogil,
+        'parallel': _parallel,
+    }
 
 
 # ----------------------
